// Ansteuerung eines HD44780 kompatiblen LCD im 4-Bit-Interfacemodus
// http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial
//
// Die Pinbelegung ist ber defines in lcd-routines.h einstellbar
 
#include "lcd-routines.h"

#include <avr/io.h>
#include <util/delay.h>

#ifdef USEEEPROM
  #include <avr/eeprom.h>
#endif
 
// LCD Befehle
 
#define CMD_Clear                0x01    // 0 s
#define CMD_Home                 0x02    // 1520 s
#define CMD_SetEntryMode         0x04    // 37 s
#define CMD_SetDisplayAndCursor  0x08    // 37 s
#define CMD_SetCursor            0x10    // 37 s
#define CMD_SetIFOptions         0x20    // 37 s
#define CMD_SetCGRAMAddress      0x40    // 37 s
#define CMD_SetDDRAMAddress      0x80    // 37 s
 

// erzeugt den Enable-Puls
static void lcd_enable(void)
{
	LCD_PORT |= _BV(LCD_EN1);
 
  _delay_us(1);                   // kurze Pause
 
   // Bei Problemen ggf. Pause gem Datenblatt des LCD Controllers verlngern
   // http://www.mikrocontroller.net/topic/80900
  LCD_PORT &= ~_BV(LCD_EN1);
}
 
void lcd_send(unsigned char AData) 
{
    // oberes nibble setzen
  LCD_PORT = (LCD_PORT & 0xf0 ) | ((AData >> 4) & 0x0f);
  _delay_us(5);
  lcd_enable();
 
   // unteres nibble setzen
  LCD_PORT = (LCD_PORT & 0xf0 ) | (AData & 0x0f);
  _delay_us(5);
  lcd_enable();
  
  _delay_us(60);  

  LCD_PORT &= 0xF0;
}

// sendet einen Befehl an das LCD
void lcd_command(unsigned char temp1)
{
   LCD_PORT &= ~_BV(LCD_RS);        // RS auf 0 setzen
   lcd_send(temp1);
}

// sendet ein Datenbyte an das LCD
void lcd_data(unsigned char temp1)
{
   LCD_PORT |= _BV(LCD_RS);        // RS auf 1 setzen
   lcd_send(temp1);
}
 
// Initialisierung: 
// Muss ganz am Anfang des Programms aufgerufen werden.
 
void lcd_init(void)
{
  LCD_DDR = LCD_DDR | 0x0F | _BV(LCD_RS) | _BV(LCD_EN1);   // Port auf Ausgang schalten
   
   // muss 3mal hintereinander gesendet werden zur Initialisierung
  _delay_ms(15);
  LCD_PORT = (LCD_PORT & 0xF0 & ~_BV(LCD_RS)) | 0x03;
//   LCD_PORT &= ~_BV(LCD_RS);      // RS auf 0

  lcd_enable();
 
  _delay_ms(5);
  lcd_enable();
 
  _delay_ms(1);
  lcd_enable();
  _delay_ms(1);

    // 4Bit / 2 Zeilen / 5x7
  lcd_command(CMD_SetIFOptions | 0x08);

    // Display ein / Cursor aus / kein Blinken
  lcd_command(CMD_SetDisplayAndCursor | 0x04);
  
    // inkrement / kein Scrollen    
  lcd_command(CMD_SetEntryMode | 0x02);				

  lcd_clear();
}
 
// Sendet den Befehl zur Lschung des Displays
 
void lcd_clear(void)
{
  lcd_command(CMD_Clear);
  _delay_ms(5);
}
 
 
// Schreibt einen String auf das LCD
 
void lcd_string(const char *data)
{
  unsigned char c;

  while ((c = *data++))
    lcd_data(c);
}

void lcd_cursor(const unsigned char ARow, const unsigned char AColumn)
{
  lcd_command(CMD_SetDDRAMAddress | AColumn | ARow ? 0x40 : 0x00);
}

void lcd_DefineChar(const unsigned char AIndex)
{
  lcd_command(CMD_SetCGRAMAddress | AIndex);
}

#ifdef USEEEPROM
//String aus EEPROM laden und an LCD senden
void lcd_eep_string(const char *data)
{	
	unsigned char c;

  while ((c = eeprom_read_byte((const unsigned char*)data++)))
    lcd_data(c);
}
#endif

